#include <fstream.h>
#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <math.h>

#define MAX_TR	      40
#define TEX_NAME	   "clock.bmp"
#define TEX_WIDTH		256
#define TEX_HEIGHT	256

UINT timer;
char *texture;

const GLfloat no_mat[] = { 0.0, 0.0, 0.0, 1.0 };
const GLfloat cover_color[] = { 0.7, 1.0, 0.3, 1.0 };
const GLfloat cover2_color[] = { 0.7, 1.0, 0.7, 1.0 };
const GLfloat glass_color[] = { 1.0, 0.7, 1.0, 0.5 };
const GLfloat arrow_color[] = { 0.0, 0.1, 0.2, 1.0 };
const GLfloat gold_color[] = { 1.0, 1.0, 0.0, 1.0 };
const GLfloat num_color[] = { 1.0, 1.0, 1.0, 1.0 };
const GLfloat surface1_color[] = { 0.2, 0.2, 0.2, 0.9 };
const GLfloat surface2_color[] = { 0.4, 0.4, 0.4, 0.9 };
const GLfloat mat_specular[] = { 0.7, 0.7, 0.7, 1.0 };
const GLfloat mat_ambient[] = { 0.5, 0.5, 0.5, 1.0 };
const GLfloat high_shininess[] = { 50.0 };
const GLfloat C = 360.0 / (GLfloat)MAX_TR * M_PI / 180.0;

void setDCPixelFormat(HDC hdc)
{
  static PIXELFORMATDESCRIPTOR pfd =
  {
    sizeof(PIXELFORMATDESCRIPTOR),          // sizeof this structure
    1,                                      // version number
    PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL
    | PFD_DOUBLEBUFFER,                     // flags
    PFD_TYPE_RGBA,                          // RGBA pixel values
    16,                                     // 24-bit color
    0, 0, 0, 0, 0, 0,                       // don't care about these
    0, 0,                                   // no alpha buffer
    0, 0, 0, 0, 0,                          // no accumulation buffer
    16,                                     // 32-bit depth buffer
    0,                                      // no stencil buffer
    0,                                      // no auxiliary buffers
    PFD_MAIN_PLANE,                         // layer type
    0,                                      // reserved(must be 0)
    0, 0, 0                                 // no layer masks
  };
  int pixelFormat;

  pixelFormat = ChoosePixelFormat(hdc, &pfd);
  SetPixelFormat(hdc, pixelFormat, &pfd);
  DescribePixelFormat(hdc, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
}

void InitializeGL()
{
  GLfloat ambient[] = { 0.0, 0.0, 0.0, 1.0 };
  GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat position[] = { 0.0, 0.0, 5.0, 0.0 };
  GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };
  GLfloat local_view[] = { 0.0 };

  glFrontFace(GL_CW);
  glCullFace(GL_BACK);
  glEnable(GL_CULL_FACE);
  glEnable(GL_SMOOTH);

  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
  glClearColor(0.3, 0.8, 1.0, 0.0);

  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, position);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_WIDTH, TEX_HEIGHT, 0, GL_RGB, GL_UNSIGNED_BYTE, texture);
}


void DrawStrip(GLfloat r1, GLfloat r2, GLfloat z1, GLfloat z2, GLfloat xn, GLfloat zn, GLfloat xn2, GLfloat zn2, int n)
{
  GLfloat x = 1.0, y = 0.0;
  GLfloat C = 2.0 / n * M_PI;

  glBegin(GL_QUAD_STRIP);
    glNormal3f(x * xn2, y * xn2, zn2);
    glVertex3f(r2, 0, z2);
    glNormal3f(x * xn, y * xn, zn);
    glVertex3f(r1, 0, z1);
    for (int i = 0; i <= n; i++)
    {
      x = cos(i * C);
      y = sin(i * C);
      glNormal3f(x * xn2, y * xn2, zn2);
      glVertex3f(x * r2, y * r2, z2);
      glNormal3f(x * xn, y * xn, zn);
      glVertex3f(x * r1, y * r1, z1);
    }
  glEnd();
}

void DrawRoundStrip(GLfloat r1, GLfloat r, GLfloat z, int n, int m)
{
  GLfloat C = 1.0 / n * M_PI, x, y, last_x = r1, last_y = 0.0;
  GLfloat n__x = 0.0, n__z = -1.0;

  for (int i = 0; i <= m; i++)
  {
    x = -cos(i * C + M_PI * 0.5);
    y = sin(i * C + M_PI * 0.5);
    DrawStrip(r1 + last_x * r, r1 + x * r, last_y * r + z, y * r + z, n__x, n__z, x, y, MAX_TR);
    last_x = x;
    last_y = y;
    n__x = x;
    n__z = y;
  }
}

void DrawNumpad(GLfloat r, GLfloat z, GLfloat tex)
{
  GLfloat x = r, y = 0.0, last_x = x * r, last_y = y;

  glEnable(GL_TEXTURE_2D);
  glBegin(GL_TRIANGLES);
    glNormal3f(0.0, 0.0, 1.0);
    for (int i = 1; i <= MAX_TR; i++)
    {
      x = cos(i * C);
      y = sin(i * C);
      glTexCoord2f(0.5, 0.5 * tex);
      glVertex3f(0.0, 0.0, z);
      glTexCoord2f((x + 1) * 0.5, (y + 1) * 0.5 * tex);
      glVertex3f(x * r, y * r, z);
      glTexCoord2f((last_x + 1) * 0.5, (last_y + 1) * 0.5 * tex);
      glVertex3f(last_x, last_y, z);
      last_x = x * r;
      last_y = y * r;
    }
  glEnd();
  glDisable(GL_TEXTURE_2D);
}

void DrawSecArrow(GLfloat z)
{
  glMaterialfv(GL_FRONT, GL_DIFFUSE, arrow_color);
  DrawStrip(0.01, 0.05, z, z, 0.0, 1.0, 0.0, 1.0, MAX_TR >> 2);
  glPushMatrix();
  glTranslatef(0.0, -0.15, 0.0);
  DrawStrip(0.0, 0.03, z, z, 0.0, 1.0, 0.0, 1.0, MAX_TR >> 2);
  glPopMatrix();
  glBegin(GL_QUADS);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(-0.01, -0.15, z);
    glVertex3f(-0.01, -0.02, z);
    glVertex3f(0.01, -0.02, z);
    glVertex3f(0.01, -0.15, z);

    glVertex3f(-0.01, 0.02, z);
    glVertex3f(-0.01, 0.57, z);
    glVertex3f(0.01, 0.57, z);
    glVertex3f(0.01, 0.02, z);

    glVertex3f(-0.01, 0.63, z);
    glVertex3f(-0.01, 0.8, z);
    glVertex3f(0.01, 0.8, z);
    glVertex3f(0.01, 0.63, z);
  glEnd();
  glPushMatrix();
  glTranslatef(0.0, 0.6, 0.0);
  DrawStrip(0.025, 0.04, z, z, 0.0, 1.0, 0.0, 1.0, MAX_TR >> 2);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, gold_color);
  DrawStrip(0.0, 0.025, z, z, 0.0, 1.0, 0.0, 0.0, MAX_TR >> 2);
  glPopMatrix();
}

void DrawMinArrow(GLfloat z)
{
  glMaterialfv(GL_FRONT, GL_DIFFUSE, arrow_color);
  glBegin(GL_QUADS);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(-0.02, 0.02, z);
    glVertex3f(-0.06, 0.4, z);
    glVertex3f(-0.04, 0.4, z);
    glVertex3f(0.0, 0.02, z);

    glVertex3f(-0.06, 0.4, z);
    glVertex3f(0.0, 0.7, z);
    glVertex3f(0.0, 0.60, z);
    glVertex3f(-0.04, 0.4, z);

    glVertex3f(0.0, 0.7, z);
    glVertex3f(0.06, 0.4, z);
    glVertex3f(0.04, 0.4, z);
    glVertex3f(0.0, 0.60, z);

    glVertex3f(0.04, 0.4, z);
    glVertex3f(0.06, 0.4, z);
    glVertex3f(0.02, 0.02, z);
    glVertex3f(0.0, 0.02, z);
  glEnd();
  glMaterialfv(GL_FRONT, GL_DIFFUSE, gold_color);
  glBegin(GL_TRIANGLES);
    glNormal3f(0.0, 0.0, 0.0);
    glVertex3f(0.0, 0.02, z);
    glVertex3f(-0.04, 0.4, z);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(0.0, 0.4, z);

    glNormal3f(0.0, 0.0, 0.0);
    glVertex3f(0.04, 0.4, z);
    glVertex3f(0.0, 0.02, z);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(0.0, 0.4, z);

    glNormal3f(0.0, 0.0, 0.0);
    glVertex3f(-0.04, 0.4, z);
    glVertex3f(0.0, 0.6, z);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(0.0, 0.4, z);

    glNormal3f(0.0, 0.0, 0.0);
    glVertex3f(0.0, 0.6, z);
    glVertex3f(0.04, 0.4, z);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(0.0, 0.4, z);
  glEnd();
}

void DrawHourArrow(GLfloat z)
{
  glMaterialfv(GL_FRONT, GL_DIFFUSE, arrow_color);
  DrawStrip(0.01, 0.05, z, z, 0.0, 1.0, 0.0, 1.0, MAX_TR >> 2);
  glBegin(GL_QUADS);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(-0.03, 0.01, z);
    glVertex3f(-0.04, 0.12, z);
    glVertex3f(0.04, 0.12, z);
    glVertex3f(0.03, 0.01, z);

    glVertex3f(-0.06, 0.28, z);
    glVertex3f(0.0, 0.40, z);
    glVertex3f(0.0, 0.36, z);
    glVertex3f(-0.04, 0.28, z);

    glVertex3f(0.0, 0.40, z);
    glVertex3f(0.06, 0.28, z);
    glVertex3f(0.04, 0.28, z);
    glVertex3f(0.0, 0.36, z);
  glEnd();
  glMaterialfv(GL_FRONT, GL_DIFFUSE, gold_color);
  glBegin(GL_TRIANGLES);
    glNormal3f(0.0, 0.0, 0.0);
    glVertex3f(0.04, 0.28, z);
    glVertex3f(-0.04, 0.28, z);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(0.0, 0.33, z);

    glNormal3f(0.0, 0.0, 0.0);
    glVertex3f(-0.04, 0.28, z);
    glVertex3f(0.0, 0.36, z);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(0.0, 0.33, z);

    glNormal3f(0.0, 0.0, 0.0);
    glVertex3f(0.0, 0.36, z);
    glVertex3f(0.04, 0.28, z);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(0.0, 0.33, z);
  glEnd();
  glPushMatrix();
  glTranslatef(0.0, 0.2, 0.0);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, arrow_color);
  DrawStrip(0.08, 0.1, z, z, 0.0, 1.0, 0.0, 1.0, MAX_TR >> 1);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, gold_color);
  DrawStrip(0.0, 0.08, z, z, 0.0, 1.0, 0.0, 0.0, MAX_TR >> 1);
  glPopMatrix();
}

void DrawGlass(GLfloat r, GLfloat r2, GLfloat z)
{
  GLfloat x, y, last_x = cos(C * 0.5), last_y = sin(C * 0.5);
  GLfloat n_x, n_z, last_n_x = 0.0, last_n_z = 1.0;

  glEnable(GL_BLEND);
  for (int i = 1; i < MAX_TR; i++)
  {
    x = cos(i * C * 0.5);
    y = sin(i * C * 0.5);
    n_x = -y;
    n_z = 1.0 - x * r2;
    DrawStrip(x * r, last_x * r, y * r2 + z, last_y * r2 + z, n_x, n_z, last_n_x, last_n_z, MAX_TR);
    last_x = x;
    last_y = y;
    last_n_x = n_x;
    last_n_z = n_z;
  }
  glDisable(GL_BLEND);
}


void DrawClock(SYSTEMTIME time, GLfloat tex)
{
  glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
  glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, num_color);
  DrawNumpad(1.0, 0.04, tex);

  glMaterialfv(GL_FRONT, GL_DIFFUSE, cover_color);
  DrawRoundStrip(1.0, 0.1, 0.0, MAX_TR >> 2, MAX_TR >> 2);
  DrawStrip(1.0, 1.0, -0.1, 0.1, -1.0, 0.0, -1.0, 0.0, MAX_TR);

  glMaterialfv(GL_FRONT, GL_DIFFUSE, cover2_color);
  DrawStrip(1.0, 1.0, -0.1, -0.12, 1.0, 0.0, 1.0, 0.0, MAX_TR);
  DrawStrip(1.0, 0.7, -0.12, -0.12, 0.0, -1.0, 0.0, -1.0, MAX_TR);
  DrawStrip(0.7, 0.65, -0.12, -0.1, -0.37, -0.93, -0.37, -0.93, MAX_TR);
  DrawStrip(0.65, 0.0, -0.1, -0.1, 0.0, -1.0, 0.0, -1.0, MAX_TR);

  glPushMatrix();
  glRotatef(90 - tex * 90 -tex * time.wSecond * 6.0, 0.0, 0.0, 1.0);
  glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat);
  glMaterialfv(GL_FRONT, GL_SPECULAR, arrow_color);
  DrawSecArrow(0.055);
  glPopMatrix();

  glPushMatrix();
  glRotatef(90 - tex * 90 -tex * (time.wMinute * 6.0 + time.wSecond * 0.1), 0.0, 0.0, 1.0);
  DrawMinArrow(0.05);
  glPopMatrix();

  glPushMatrix();
  glRotatef(90 - tex * 90 -tex * ((time.wHour + 4) * 30.0 + time.wMinute * 0.5), 0.0, 0.0, 1.0);
  DrawHourArrow(0.045);
  glPopMatrix();

  glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, glass_color);
  DrawGlass(1.0, 0.2, 0.09);
}

void DrawElement(GLfloat x, GLfloat y)
{
  glMaterialfv(GL_FRONT, GL_DIFFUSE, surface1_color);
  glBegin(GL_QUADS);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(x, 0, y);
    glVertex3f(x + 0.5, 0, y);
    glVertex3f(x + 0.5, 0, y + 0.5);
    glVertex3f(x, 0, y + 0.5);

    glVertex3f(x + 0.5, 0, y + 0.5);
    glVertex3f(x + 1.0, 0, y + 0.5);
    glVertex3f(x + 1.0, 0, y + 1.0);
    glVertex3f(x + 0.5, 0, y + 1.0);
  glEnd();
  glMaterialfv(GL_FRONT, GL_DIFFUSE, surface2_color);
  glBegin(GL_QUADS);
    glNormal3f(0.0, 0.0, 1.0);
    glVertex3f(x + 0.5, 0, y);
    glVertex3f(x + 1.0, 0, y);
    glVertex3f(x + 1.0, 0, y + 0.5);
    glVertex3f(x + 0.5, 0, y + 0.5);

    glVertex3f(x, 0, y + 0.5);
    glVertex3f(x + 0.5, 0, y + 0.5);
    glVertex3f(x + 0.5, 0, y + 1.0);
    glVertex3f(x, 0, y + 1.0);
  glEnd();
}

void drawScene(HDC hDC)
{
  static int angle = 0;
  SYSTEMTIME time;

  GetSystemTime(&time);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glTranslatef(0.0, -0.5, 0.0);

  glPushMatrix();
  glTranslatef(0.0, 1.1, -8.0);
  glRotatef(angle, 1.0, 0.0, 0.0);
  glRotatef(angle, 0.0, 0.0, 1.0);
  glRotatef(angle / 2, 0.0, 1.0, 0.0);
  DrawClock(time, 1);
  glPopMatrix();

  glPushMatrix();
  glTranslatef(0.0, -1.1, -8.0);
  glRotatef(-angle, 1.0, 0.0, 0.0);
  glRotatef(-angle, 0.0, 0.0, 1.0);
  glRotatef(angle / 2, 0.0, 1.0, 0.0);
  DrawClock(time, -1);
  glPopMatrix();

  glEnable(GL_BLEND);
  glMaterialfv(GL_FRONT, GL_AMBIENT, no_mat);

  for (int i = -4; i < 4; i++)
    for (int j = -10; j < 1; j++)
      DrawElement(i, j);

  SwapBuffers(hDC);
  glFlush();

  angle += 4;
}

LONG WINAPI wndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static HDC hdc;
  static HGLRC hrc;
  PAINTSTRUCT ps;
  GLdouble aspect;
  GLsizei width, height;

  switch(msg)
  {
    case WM_CREATE:
      hdc = GetDC(hWnd);
      setDCPixelFormat(hdc);
      hrc = wglCreateContext(hdc);
      wglMakeCurrent(hdc, hrc);
      InitializeGL();
      timer = SetTimer(hWnd, 1, 50, NULL);
      return 0;

    case WM_SIZE:
      width = (GLsizei)LOWORD(lParam);
      height = (GLsizei)HIWORD(lParam);
      if (!width || !height) return 0;

      aspect = (GLdouble)width /(GLdouble)height;
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(30.0, aspect, 1.0, 10.0);
      glViewport(0, 0, width, height);

      return 0;

    case WM_PAINT:
      BeginPaint(hWnd, &ps);
      drawScene(hdc);
      EndPaint(hWnd, &ps);
      return 0;

    case WM_DESTROY:
      wglMakeCurrent(NULL, NULL);
      wglDeleteContext(hrc);
      KillTimer(hWnd, timer);
      PostQuitMessage(0);
      return 0;

    case WM_TIMER:
      InvalidateRect(hWnd, NULL, FALSE);
      return 0;
  }

  return DefWindowProc(hWnd, msg, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR cmdLine, int cmdShow)
{
  static char appName [] = "Bank - by HackMaster";
  WNDCLASS wc;
  HWND hWnd;
  MSG msg;

  texture = new char[TEX_WIDTH * TEX_HEIGHT * 3];
  ifstream in(TEX_NAME, ios::binary);
  if (!in)
  {
    MessageBox(0, " DOLLAR.BMP  ", "", MB_OK | MB_ICONSTOP);
    return 1;
  }
  in.seekg(0x36, ios::beg);
  in.read(texture, TEX_WIDTH * TEX_HEIGHT * 3);
  in.close();

  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc =(WNDPROC)wndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground =(HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszMenuName = (LPCTSTR)0;
  wc.lpszClassName = appName;

  RegisterClass(&wc);

  hWnd = CreateWindow(appName, appName,
                      WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                      CW_USEDEFAULT, CW_USEDEFAULT, 320,
                      240, HWND_DESKTOP, NULL, hInstance, NULL);

  ShowWindow(hWnd, cmdShow);
  UpdateWindow(hWnd);

  while(GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  return msg.wParam;
}
